@@ -107,6 +107,7 @@ class Alias(object):
         Run the API call `update_aliases` with the results of `body()`
         """
         self.loggit.info('Updating aliases...')
+        self.loggit.info('Alias actions: {0}'.format(self.body()))
         try:
             self.client.indices.update_aliases(body=self.body())
         except Exception as e:
@@ -153,10 +154,8 @@ class Allocation(object):
         #: Instance variable.
         #: Populated at instance creation time. Value is
         #: ``index.routing.allocation.`` `allocation_type` ``.`` `key` ``.`` `value`
-        self.body       = (
-            'index.routing.allocation.'
-            '{0}.{1}={2}'.format(allocation_type, key, value)
-        )
+        bkey = 'index.routing.allocation.{0}.{1}'.format(allocation_type, key)
+        self.body       = { bkey : value }
         #: Instance variable.
         #: Internal reference to `wait_for_completion`
         self.wfc        = wait_for_completion
@@ -176,7 +175,7 @@ class Allocation(object):
         Change allocation settings for indices in `index_list.indices` with the
         settings in `body`.
         """
-        self.loggit.info(
+        self.loggit.debug(
             'Cannot get change shard routing allocation of closed indices.  '
             'Omitting any closed indices.'
         )
@@ -191,12 +190,12 @@ class Allocation(object):
                     index=to_csv(l), body=self.body
                 )
                 if self.wfc:
-                    logger.info(
+                    logger.debug(
                         'Waiting for shards to complete relocation for indices:'
                         ' {0}'.format(to_csv(l))
                     )
                     self.client.cluster.health(index=to_csv(l),
-                        level='indices', wait_for_relocation_shards=0,
+                        level='indices', wait_for_relocating_shards=0,
                         timeout=self.timeout,
                     )
         except Exception as e:
@@ -236,7 +235,8 @@ class Close(object):
         """
         self.index_list.filter_closed()
         self.index_list.empty_list_check()
-        self.loggit.info('Closing selected indices')
+        self.loggit.info(
+            'Closing selected indices: {0}'.format(self.index_list.indices))
         try:
             index_lists = chunk_index_list(self.index_list.indices)
             for l in index_lists:
@@ -244,8 +244,14 @@ class Close(object):
                     self.loggit.info(
                         'Deleting aliases from indices before closing.')
                     self.loggit.debug('Deleting aliases from: {0}'.format(l))
-                    self.client.indices.delete_alias(
-                        index=to_csv(l), name='_all')
+                    try:
+                        self.client.indices.delete_alias(
+                            index=to_csv(l), name='_all')
+                    except Exception as e:
+                        self.loggit.warn(
+                            'Some indices may not have had aliases.  Exception:'
+                            ' {0}'.format(e)
+                        )
                 self.client.indices.flush(
                     index=to_csv(l), ignore_unavailable=True)
                 self.client.indices.close(
@@ -253,6 +259,93 @@ class Close(object):
         except Exception as e:
             report_failure(e)
 
+class ClusterRouting(object):
+    def __init__(
+        self, client, routing_type=None, setting=None, value=None,
+        wait_for_completion=False, timeout=30,
+    ):
+        """
+        For now, the cluster routing settings are hardcoded to be ``transient``
+
+        :arg client: An :class:`elasticsearch.Elasticsearch` client object
+        :arg routing_type: Type of routing to apply. Either `allocation` or
+            `rebalance`
+        :arg setting: Currently, the only acceptable value for `setting` is
+            ``enable``. This is here in case that changes.
+        :arg value: Used only if `setting` is `enable`. Semi-dependent on
+            `routing_type`. Acceptable values for `allocation` and `rebalance`
+            are ``all``, ``primaries``, and ``none`` (string, not `NoneType`).
+            If `routing_type` is `allocation`, this can also be
+            ``new_primaries``, and if `rebalance`, it can be ``replicas``.
+        :arg wait_for_completion: Wait (or not) for the operation
+            to complete before returning.  (default: `False`)
+        :type wait_for_completion: bool
+        :arg timeout: Number of seconds to `wait_for_completion`
+        """
+        verify_client_object(client)
+        #: Instance variable.
+        #: An :class:`elasticsearch.Elasticsearch` client object
+        self.client  = client
+        self.loggit  = logging.getLogger('curator.actions.cluster_routing')
+        #: Instance variable.
+        #: Internal reference to `wait_for_completion`
+        self.wfc     = wait_for_completion
+        #: Instance variable.
+        #: How long in seconds to `wait_for_completion` before returning with an
+        #: exception
+        self.timeout = '{0}s'.format(timeout)
+
+        if setting != 'enable':
+            raise ValueError(
+                'Invalid value for "setting": {0}.'.format(setting)
+            )
+        if routing_type == 'allocation':
+            if value not in ['all', 'primaries', 'new_primaries', 'none']:
+                raise ValueError(
+                    'Invalid "value": {0} with "routing_type":'
+                    '{1}.'.format(value, routing_type)
+                )
+        elif routing_type == 'rebalance':
+            if value not in ['all', 'primaries', 'replicas', 'none']:
+                raise ValueError(
+                    'Invalid "value": {0} with "routing_type":'
+                    '{1}.'.format(value, routing_type)
+                )
+        else:
+            raise ValueError(
+                'Invalid value for "routing_type": {0}.'.format(routing_type)
+            )
+        bkey = 'cluster.routing.{0}.{1}'.format(routing_type,setting)
+        self.body = { 'transient' : { bkey : value } }
+
+    def do_dry_run(self):
+        """
+        Log what the output would be, but take no action.
+        """
+        logger.info('DRY-RUN MODE.  No changes will be made.')
+        self.loggit.info(
+            'DRY-RUN: Update cluster routing settings with arguments: '
+            '{0}'.format(self.body)
+        )
+
+    def do_action(self):
+        """
+        Change cluster routing settings with the settings in `body`.
+        """
+        self.loggit.info('Updating cluster settings: {0}'.format(self.body))
+        try:
+            self.client.cluster.put_settings(body=self.body)
+            if self.wfc:
+                logger.debug(
+                    'Waiting for shards to complete routing and/or rebalancing'
+                )
+                self.client.cluster.health(
+                    level='indices', wait_for_relocating_shards=0,
+                    timeout=self.timeout,
+                )
+        except Exception as e:
+            report_failure(e)
+
 class CreateIndex(object):
     def __init__(self, client, name, extra_settings={}):
         """
@@ -381,7 +474,8 @@ class DeleteIndices(object):
         Delete indices in `index_list.indices`
         """
         self.index_list.empty_list_check()
-        self.loggit.info('Deleting selected indices')
+        self.loggit.info(
+            'Deleting selected indices: {0}'.format(self.index_list.indices))
         try:
             index_lists = chunk_index_list(self.index_list.indices)
             for l in index_lists:
@@ -477,7 +571,8 @@ class Open(object):
         Open closed indices in `index_list.indices`
         """
         self.index_list.empty_list_check()
-        self.loggit.info('Opening selected indices')
+        self.loggit.info(
+            'Opening selected indices: {0}'.format(self.index_list.indices))
         try:
             index_lists = chunk_index_list(self.index_list.indices)
             for l in index_lists:
@@ -529,22 +624,22 @@ class Replicas(object):
         Update the replica count of indices in `index_list.indices`
         """
         self.index_list.empty_list_check()
-        self.loggit.info(
+        self.loggit.debug(
             'Cannot get update replica count of closed indices.  '
             'Omitting any closed indices.'
         )
         self.index_list.filter_closed()
         self.loggit.info(
-            'Updating the replica count of selected indices to '
-            '{0}'.format(self.count)
+            'Setting the replica count to {0} for indices: '
+            '{1}'.format(self.count, self.index_list.indices)
         )
         try:
             index_lists = chunk_index_list(self.index_list.indices)
             for l in index_lists:
                 self.client.indices.put_settings(index=to_csv(l),
-                    body='number_of_replicas={0}'.format(self.count))
+                    body={'number_of_replicas' : self.count})
                 if self.wfc and self.count > 0:
-                    logger.info(
+                    logger.debug(
                         'Waiting for shards to complete replication for '
                         'indices: {0}'.format(to_csv(l))
                     )
@@ -645,6 +740,9 @@ class Snapshot(object):
         :type skip_repo_fs_check: bool
         """
         verify_index_list(ilo)
+        # Check here and don't bother with the rest of this if there are no
+        # indices in the index list.
+        ilo.empty_list_check()
         if not repository_exists(ilo.client, repository=repository):
             raise ActionError(
                 'Cannot snapshot indices to missing repository: '
@@ -732,6 +830,9 @@ class Snapshot(object):
         if snapshot_running(self.client):
             raise SnapshotInProgress('Snapshot already in progress.')
         try:
+            self.loggit.info('Creating snapshot "{0}" from indices: '
+                '{1}'.format(self.name, self.index_list.indices)
+            )
             self.client.snapshot.create(
                 repository=self.repository, snapshot=self.name, body=self.body,
                 wait_for_completion=self.wait_for_completion
@@ -852,7 +953,7 @@ class Restore(object):
                 'rename_replacement' : self.rename_replacement,
             }
         if extra_settings:
-            self.loggit.info(
+            self.loggit.debug(
                 'Adding extra_settings to restore body: '
                 '{0}'.format(extra_settings)
             )
@@ -948,6 +1049,9 @@ class Restore(object):
             raise SnapshotInProgress(
                 'Cannot restore while a snapshot is in progress.')
         try:
+            self.loggit.info('Restoring indices "{0}" from snapshot: '
+                '{1}'.format(self.indices, self.name)
+            )
             self.client.snapshot.restore(
                 repository=self.repository, snapshot=self.name, body=self.body,
                 wait_for_completion=self.wfc
