@@ -1,13 +1,12 @@
 from datetime import timedelta, datetime, date
 import elasticsearch
-import copy
 import time
-import re
-import sys
 import logging
-import yaml
+import yaml, os, re, sys
+from voluptuous import Schema
 from .exceptions import *
-from .settings import DATE_REGEX, VERSION_MAX, VERSION_MIN, ACTION_DEFAULTS
+from .defaults import settings
+from .validators import SchemaCheck, actions, filters, options
 from ._version import __version__
 logger = logging.getLogger(__name__)
 
@@ -34,6 +33,21 @@ def get_yaml(path):
     :arg path: The path to a YAML configuration file.
     :rtype: dict
     """
+    # Set the stage here to parse single scalar value environment vars from
+    # the YAML file being read
+    single = re.compile( r'^\$\{(.*)\}$' )
+    yaml.add_implicit_resolver ( "!single", single )
+    def single_constructor(loader,node):
+        value = loader.construct_scalar(node)
+        proto = single.match(value).group(1)
+        default = None
+        if len(proto.split(':')) > 1:
+            envvar, default = proto.split(':')
+        else:
+            envvar = proto
+        return os.environ[envvar] if envvar in os.environ else default
+    yaml.add_constructor('!single', single_constructor)
+
     raw = read_file(path)
     try:
         cfg = yaml.load(raw)
@@ -129,8 +143,8 @@ def get_date_regex(timestring):
         curr = timestring[s]
         if curr == '%':
             pass
-        elif curr in DATE_REGEX and prev == '%':
-            regex += '\d{' + DATE_REGEX[curr] + '}'
+        elif curr in settings.date_regex() and prev == '%':
+            regex += '\d{' + settings.date_regex()[curr] + '}'
         elif curr in ['.', '-']:
             regex += "\\" + curr
         else:
@@ -160,13 +174,6 @@ def get_datetime(index_timestamp, timestring):
         if not '%d' in timestring:
             timestring += '%d'
             index_timestamp += '1'
-    # logger.debug(
-    #     'index_timestamp: {0}, timestring: {1}, return value: '
-    #     '{2}'.format(
-    #         index_timestamp, timestring,
-    #         datetime.strptime(index_timestamp, timestring)
-    #     )
-    # )
     return datetime.strptime(index_timestamp, timestring)
 
 def fix_epoch(epoch):
@@ -367,6 +374,24 @@ def get_indices(client):
             client.indices.get_settings(
             index='_all', params={'expand_wildcards': 'open,closed'})
         )
+        version_number = get_version(client)
+        logger.debug(
+            'Detected Elasticsearch version '
+            '{0}'.format(".".join(map(str,version_number)))
+        )
+        # This hack ONLY works if you're using 2.4.2 or higher, but is unneeded
+        # if you are using 5.0 or higher.  See issue #826
+        if version_number >= (2, 4, 2) \
+            and version_number < (5, 0, 0):
+            logger.debug('Using Elasticsearch >= 2.4.2 < 5.0.0')
+            if client.indices.exists(index='.security'):
+                logger.debug(
+                    'Found the ".security" index.  '
+                    'Adding to list of all indices'
+                )
+                # Double check to see if it's there before appending
+                if not '.security' in indices:
+                    indices.append('.security')
         logger.debug("All indices: {0}".format(indices))
         return indices
     except Exception as e:
@@ -412,7 +437,8 @@ def check_version(client):
         'Detected Elasticsearch version '
         '{0}'.format(".".join(map(str,version_number)))
     )
-    if version_number >= VERSION_MAX or version_number < VERSION_MIN:
+    if version_number >= settings.version_max() \
+        or version_number < settings.version_min():
         raise CuratorException(
             'Elasticsearch version {0} incompatible '
             'with this version of Curator '
@@ -436,6 +462,12 @@ def check_master(client, master_only=False):
 
 def get_client(**kwargs):
     """
+    NOTE: AWS IAM parameters `aws_key`, `aws_secret_key`, and `aws_region` are
+    provided for future compatibility, should AWS ES support the
+    ``/_cluster/state/metadata`` endpoint.  So long as this endpoint does not
+    function in AWS ES, the client will not be able to use
+    :class:`curator.indexlist.IndexList`, which is the backbone of Curator 4
+
     Return an :class:`elasticsearch.Elasticsearch` client object using the
     provided parameters. Any of the keyword arguments the
     :class:`elasticsearch.Elasticsearch` client object can receive are valid,
@@ -481,6 +513,12 @@ def get_client(**kwargs):
                 kwargs['url_prefix'] == "None"
             ):
             kwargs['url_prefix'] = ''
+    if 'host' in kwargs and 'hosts' in kwargs:
+        raise ConfigurationError(
+            'Both "host" and "hosts" are defined.  Pick only one.')
+    elif 'host' in kwargs and not 'hosts' in kwargs:
+        kwargs['hosts'] = kwargs['host']
+        del kwargs['host']
     kwargs['hosts'] = '127.0.0.1' if not 'hosts' in kwargs else kwargs['hosts']
     kwargs['master_only'] = False if not 'master_only' in kwargs \
         else kwargs['master_only']
@@ -500,32 +538,37 @@ def get_client(**kwargs):
         if kwargs['ssl_no_validate']:
             kwargs['verify_certs'] = False # Not needed, but explicitly defined
         else:
-            logger.info('Attempting to verify SSL certificate.')
+            logger.debug('Attempting to verify SSL certificate.')
             # If user provides a certificate:
             if kwargs['certificate']:
                 kwargs['verify_certs'] = True
                 kwargs['ca_certs'] = kwargs['certificate']
-            else: # Try to use certifi certificates:
-                try:
+            else: # Try to use bundled certifi certificates
+                if getattr(sys, 'frozen', False):
+                    # The application is frozen (compiled)
+                    datadir = os.path.dirname(sys.executable)
+                    kwargs['verify_certs'] = True
+                    kwargs['ca_certs'] = os.path.join(datadir, 'cacert.pem')
+                else:
+                    # Use certifi certificates via certifi.where():
                     import certifi
                     kwargs['verify_certs'] = True
                     kwargs['ca_certs'] = certifi.where()
-                except ImportError:
-                    logger.warn('Unable to verify SSL certificate.')
+
     try:
         from requests_aws4auth import AWS4Auth
         kwargs['aws_key'] = False if not 'aws_key' in kwargs \
             else kwargs['aws_key']
         kwargs['aws_secret_key'] = False if not 'aws_secret_key' in kwargs \
             else kwargs['aws_secret_key']
-        kwargs['region'] = False if not 'region' in kwargs \
-            else kwargs['region']
-        if kwargs['aws_key'] or kwargs['aws_secret_key'] or kwargs['region']:
+        kwargs['aws_region'] = False if not 'aws_region' in kwargs \
+            else kwargs['aws_region']
+        if kwargs['aws_key'] or kwargs['aws_secret_key'] or kwargs['aws_region']:
             if not kwargs['aws_key'] and kwargs['aws_secret_key'] \
-                    and kwargs['region']:
+                    and kwargs['aws_region']:
                 raise MissingArgument(
                     'Missing one or more of "aws_key", "aws_secret_key", '
-                    'or "region".'
+                    'or "aws_region".'
                 )
             # Override these kwargs
             kwargs['use_ssl'] = True
@@ -534,7 +577,7 @@ def get_client(**kwargs):
             kwargs['http_auth'] = (
                 AWS4Auth(
                     kwargs['aws_key'], kwargs['aws_secret_key'],
-                    kwargs['region'], 'es')
+                    kwargs['aws_region'], 'es')
             )
         else:
             logger.debug('"requests_aws4auth" module present, but not used.')
@@ -576,7 +619,7 @@ def override_timeout(timeout, action):
                 retval = 21600
             elif action == 'sync_flush':
                 retval = 180
-            logger.info(
+            logger.debug(
                 'Overriding default connection timeout for {0} action.  '
                 'New timeout: {1}'.format(action.upper(),timeout)
             )
@@ -737,7 +780,7 @@ def create_snapshot_body(indices, ignore_unavailable=False,
     :rtype: dict
     """
     if not indices:
-        logger.error('Missing required repository parameter')
+        logger.error('No indices provided.')
         return False
     body = {
         "ignore_unavailable": ignore_unavailable,
@@ -796,21 +839,21 @@ def create_repo_body(repo_type=None,
     body = {}
     body['type'] = argdict['repo_type']
     body['settings'] = {}
-    settings = []
+    settingz = [] # Differentiate from module settings
     maybes   = [
                 'compress', 'chunk_size',
                 'max_restore_bytes_per_sec', 'max_snapshot_bytes_per_sec'
                ]
     s3       = ['bucket', 'region', 'base_path', 'access_key', 'secret_key']
 
-    settings += [i for i in maybes if argdict[i]]
+    settingz += [i for i in maybes if argdict[i]]
     # Type 'fs'
     if argdict['repo_type'] == 'fs':
-        settings.append('location')
+        settingz.append('location')
     # Type 's3'
     if argdict['repo_type'] == 's3':
-        settings += [i for i in s3 if argdict[i]]
-    for k in settings:
+        settingz += [i for i in s3 if argdict[i]]
+    for k in settingz:
         body['settings'][k] = argdict[k]
     return body
 
@@ -854,20 +897,20 @@ def create_repository(client, **kwargs):
 
     try:
         body = create_repo_body(**kwargs)
-        logger.info(
+        logger.debug(
             'Checking if repository {0} already exists...'.format(repository)
         )
         result = get_repository(client, repository=repository)
         logger.debug("Result = {0}".format(result))
         if not result:
-            logger.info(
+            logger.debug(
                 'Repository {0} not in Elasticsearch. Continuing...'.format(
                     repository
                 )
             )
             client.snapshot.create_repository(repository=repository, body=body)
         elif result is not None and repository not in result:
-            logger.info(
+            logger.debug(
                 'Repository {0} not in Elasticsearch. Continuing...'.format(
                     repository
                 )
@@ -887,7 +930,7 @@ def create_repository(client, **kwargs):
                 repository, e.status_code, e.error
                 )
         )
-    logger.info("Repository {0} creation initiated...".format(repository))
+    logger.debug("Repository {0} creation initiated...".format(repository))
     return True
 
 def repository_exists(client, repository=None):
@@ -902,10 +945,10 @@ def repository_exists(client, repository=None):
         raise MissingArgument('No value for "repository" provided')
     test_result = get_repository(client, repository)
     if repository in test_result:
-        logger.info("Repository {0} exists.".format(repository))
+        logger.debug("Repository {0} exists.".format(repository))
         return True
     else:
-        logger.info("Repository {0} not found...".format(repository))
+        logger.debug("Repository {0} not found...".format(repository))
         return False
 
 def test_repo_fs(client, repository=None):
@@ -918,7 +961,7 @@ def test_repo_fs(client, repository=None):
     try:
         nodes = client.snapshot.verify_repository(
             repository=repository)['nodes']
-        logger.info('All nodes can write to the repository')
+        logger.debug('All nodes can write to the repository')
         logger.debug(
             'Nodes with verified repository access: {0}'.format(nodes))
     except Exception as e:
@@ -982,7 +1025,7 @@ def parse_date_pattern(name):
         curr = name[s]
         if curr == '%':
             pass
-        elif curr in DATE_REGEX and prev == '%':
+        elif curr in settings.date_regex() and prev == '%':
             rendered += str(datetime.utcnow().strftime('%{0}'.format(curr)))
         else:
             rendered += curr
@@ -1001,24 +1044,114 @@ def prune_nones(mydict):
     # Test for `None` instead of existence or zero values will be caught
     return dict([(k,v) for k, v in mydict.items() if v != None and v != 'None'])
 
-def verify_args(action, options):
+def validate_filters(action, filters):
     """
-    Verify that only acceptable argument names in `options` have been passed for
-    any given `action`
+    Validate that the filters are appropriate for the action type, e.g. no
+    index filters applied to a snapshot list.
 
-    :arg action: The name of an action to be performed
-    :arg options: A dictionary of options.
+    :arg action: An action name
+    :arg filters: A list of filters to test.
     """
-    logger.debug('Arguments for action "{0}": {1}'.format(action, options))
-    def matches_keys(mydict):
-        logger.debug(
-            'options.keys = {0} mydict.keys = '
-            '{1}'.format(options.keys(), mydict.keys())
-        )
-        return sorted(list(options.keys())) == sorted(list(mydict.keys()))
+    # Define which set of filtertypes to use for testing
+    if action in settings.snapshot_actions():
+        filtertypes = settings.snapshot_filtertypes()
+    else:
+        filtertypes = settings.index_filtertypes()
+    for f in filters:
+        if f['filtertype'] not in filtertypes:
+            raise ConfigurationError(
+                '"{0}" filtertype is not compatible with action "{1}"'.format(
+                    f['filtertype'],
+                    action
+                )
+            )
+    # If we get to this point, we're still valid.  Return the original list
+    return filters
 
-    # deepcopy guarantees clean copies of the defaults, and nothing getting
-    # altered in "pass by reference," which was happening in testing.
-    if not matches_keys(copy.deepcopy(ACTION_DEFAULTS[action])):
-        raise ConfigurationError(
-            'Invalid option in configuration: {0}'.format(options))
+def validate_actions(data):
+    """
+    Validate an Action configuration dictionary, as imported from actions.yml,
+    for example.
+
+    The method returns a validated and sanitized configuration dictionary.
+
+    :arg data: The configuration dictionary
+    :rtype: dict
+    """
+    # data is the ENTIRE schema...
+    clean_config = { }
+    # Let's break it down into smaller chunks...
+    # First, let's make sure it has "actions" as a key, with a subdictionary
+    root = SchemaCheck(data, actions.root(), 'Actions File', 'root').result()
+    # We've passed the first step.  Now let's iterate over the actions...
+    for action_id in root['actions']:
+        # Now, let's ensure that the basic action structure is correct, with
+        # the proper possibilities for 'action'
+        action_dict = root['actions'][action_id]
+        loc = 'Action ID "{0}"'.format(action_id)
+        valid_structure = SchemaCheck(
+            action_dict,
+            actions.structure(action_dict, loc),
+            'structure',
+            loc
+        ).result()
+        # With the basic structure validated, now we extract the action name
+        current_action = valid_structure['action']
+        # And let's update the location with the action.
+        loc = 'Action ID "{0}", action "{1}"'.format(
+            action_id, current_action)
+        clean_options = SchemaCheck(
+            prune_nones(valid_structure['options']),
+            options.get_schema(current_action),
+            'options',
+            loc
+        ).result()
+        clean_config[action_id] = {
+            'action' : current_action,
+            'description' : valid_structure['description'],
+            'options' : clean_options,
+        }
+        if current_action == 'alias':
+            add_remove = {}
+            for k in ['add', 'remove']:
+                if k in valid_structure:
+                    current_filters = SchemaCheck(
+                        valid_structure[k]['filters'],
+                        Schema(filters.Filters(current_action, location=loc)),
+                        '"{0}" filters',
+                        '{1}, "filters"'.format(k, loc)
+                    ).result()
+                    add_remove.update(
+                        {
+                            k: {
+                                'filters' : SchemaCheck(
+                                        current_filters,
+                                        Schema(
+                                            filters.Filters(
+                                                current_action,
+                                                location=loc
+                                            )
+                                        ),
+                                        'filters',
+                                        '{0}, "{1}", "filters"'.format(loc, k)
+                                    ).result()
+                                }
+                        }
+                    )
+            # Add/Remove here
+            clean_config[action_id].update(add_remove)
+        elif current_action in [ 'cluster_routing', 'create_index' ]:
+            # neither cluster_routing nor create_index should have filters
+            pass
+        else: # Filters key only appears in non-alias actions
+            valid_filters = SchemaCheck(
+                valid_structure['filters'],
+                Schema(filters.Filters(current_action, location=loc)),
+                'filters',
+                '{0}, "filters"'.format(loc)
+            ).result()
+            clean_filters = validate_filters(current_action, valid_filters)
+            clean_config[action_id].update({'filters' : clean_filters})
+
+    # if we've gotten this far without any Exceptions raised, it's valid!
+    return { 'actions' : clean_config }
