@@ -174,6 +174,7 @@ class TestChunkIndexList(TestCase):
 class TestGetIndices(TestCase):
     def test_client_exception(self):
         client = Mock()
+        client.info.return_value = {'version': {'number': '2.4.1'} }
         client.indices.get_settings.return_value = testvars.settings_two
         client.indices.get_settings.side_effect = testvars.fake_fail
         self.assertRaises(
@@ -181,14 +182,25 @@ class TestGetIndices(TestCase):
     def test_positive(self):
         client = Mock()
         client.indices.get_settings.return_value = testvars.settings_two
+        client.info.return_value = {'version': {'number': '2.4.1'} }
         self.assertEqual(
             ['index-2016.03.03', 'index-2016.03.04'],
             sorted(curator.get_indices(client))
         )
     def test_empty(self):
         client = Mock()
+        client.info.return_value = {'version': {'number': '2.4.1'} }
         client.indices.get_settings.return_value = {}
         self.assertEqual([], curator.get_indices(client))
+    def test_issue_826(self):
+        client = Mock()
+        client.info.return_value = {'version': {'number': '2.4.2'} }
+        client.indices.get_settings.return_value = testvars.settings_two
+        client.indices.exists.return_value = True
+        self.assertEqual(
+            ['.security', 'index-2016.03.03', 'index-2016.03.04'],
+            sorted(curator.get_indices(client))
+        )
 
 class TestCheckVersion(TestCase):
     def test_check_version_(self):
@@ -254,6 +266,16 @@ class TestGetClient(TestCase):
             curator.ConfigurationError,
             curator.get_client, **kwargs
         )
+    def test_host_with_hosts(self):
+        kwargs = {
+            'url_prefix': '',
+            'host' : '127.0.0.1',
+            'hosts' : ['127.0.0.2'],
+        }
+        self.assertRaises(
+            curator.ConfigurationError,
+            curator.get_client, **kwargs
+        )
     def test_certificate_logic(self):
         kwargs = { 'use_ssl' : True, 'certificate' : 'mycert.pem' }
         self.assertRaises(
@@ -297,6 +319,7 @@ class TestShowDryRun(TestCase):
     # simple code coverage run
     def test_index_list(self):
         client = Mock()
+        client.info.return_value = {'version': {'number': '2.4.1'} }
         client.indices.get_settings.return_value = testvars.settings_two
         client.cluster.state.return_value = testvars.clu_state_two
         client.indices.stats.return_value = testvars.stats_two
@@ -563,3 +586,19 @@ class TestPruneNones(TestCase):
     def test_prune_nones_without(self):
         a = {'foo':'bar'}
         self.assertEqual(a, curator.prune_nones(a))
+
+class TestValidateFilters(TestCase):
+    def test_snapshot_with_index_filter(self):
+        self.assertRaises(
+            curator.ConfigurationError,
+            curator.validate_filters,
+            'delete_snapshots',
+            [{'filtertype': 'kibana'}]
+        )
+    def test_index_with_snapshot_filter(self):
+        self.assertRaises(
+            curator.ConfigurationError,
+            curator.validate_filters,
+            'delete_indices',
+            [{'filtertype': 'state', 'state': 'SUCCESS'}]
+        )
